load("nashorn:mozilla_compat.js");
println = print;

// Change the color of models based on keyword in the [Definition] field.
importPackage(com.change_vision.jude.api.inf.model);
importPackage(com.change_vision.jude.api.inf.editor);
importPackage(com.change_vision.jude.api.inf.presentation);

// Color to change
var COLOR = "#BBCCFF";
// Keyword in definition field
var IDENTICAL_STRING = "a";

// Script
run();

function run() {
    var currentDiagram = getCurrentDiagram();
    if (currentDiagram == null) {
        println("error: No ERD is open.\n\nOpen the ER diagram。");
        return;
    }
    var presentations = getPresentations(currentDiagram);
    var chengeColorPresentations = getIdenticalDefinitionPresentations(presentations, IDENTICAL_STRING);
    setColor(chengeColorPresentations, COLOR);
}

function getCurrentDiagram() {
    try{
        var viewManager = astah.getViewManager();
        var diagramViewManager = viewManager.getDiagramViewManager();
        return diagramViewManager.getCurrentDiagram();
    } catch(e) {
        return null;
    }
}

function getPresentations(currentDiagram) {
    try {
        return currentDiagram.getPresentations();
    } catch (e) {
        return new IPresentation[0];
    }
}

function getIdenticalDefinitionPresentations(presentations, identicalString) {
    var regularExpression = new RegExp("(.*)(" + identicalString + ")(.*)");
    var identicalDefinitionPresentations = [];
    for(var i in presentations) {
        var presentation = presentations[i];
        if (presentation == null) {
            continue;
        }
        var model = presentation.getModel();
        if (model == null) {
            continue;
        }
        if (!(model instanceof INamedElement)) {
            continue;
        }
        var definition = model.getDefinition();
        if (definition.match(regularExpression)) {
            identicalDefinitionPresentations.push(presentation);
        }
    }
    return identicalDefinitionPresentations;
}

function setColor(presentations, color) {
    try {
        TransactionManager.beginTransaction();
        for(var i in presentations) {
            var presentation = presentations[i];
            presentation.setProperty(PresentationPropertyConstants.Key.FILL_COLOR, color);
        }
        TransactionManager.endTransaction();
        println("info: " + presentations.length + " color changed.");
    } catch (e) {
        TransactionManager.abortTransaction();
        println("error: could not change.");
        println(e);
    }
}
